% Jim Wang
% 9/9/08
%
% Katie Ward
% 9/23/08
%
% Simplification of heas2 used to reproduce the conductance-based part of
% figure 3.
%
% heas2simp(dt,T,G)
%
% dt = time step (ms)
% T = total time of simulation (ms)
% G = peak synaptic conductance for exc. to exc. network synapses (mS/cm^2)
%
% Example: heas2(.01,100,0.04*1e-3)


function spkfreq = heas2simp(dt,T,G)

close all

v0 = -73;  %mV

Cm = 1;         % �F/cm^2

gbar_na = 100;  % mS/cm^2
gbar_k = 40;    % mS/cm^2
g_L = .05;

E_na = 55;      % mV
E_k = -80;      % mV
E_L = -65;      % mV

phi = 0.1;

tau_A = 20;  %ms
gbar_A = 20;  %mS/cm^2


Ein = -80;  %mV
Eex = 0;  %mV
tau = 5;  %ms  
Einp = Eex;


% initializing
Ne_net = 1000;
Ni_net = 0;

ee = zeros(Ne_net,1);
Mee = zeros(Ne_net,Ne_net);

net_E = struct('v', ee+v0, ...
    'Esyn', Eex, ...
    'gE', ee, ...
    'tau_gE', tau, ...
    'spks', [], ...
    'N', Ne_net, ...
    'h', ee, ...
    'n', ee, ...
    'b', ee, ...
    'm', ee, ...
    'finp', 1570, ...
    'theta', ee, ...
    'lambda', 5);

net_E.theta = (-pi/2) + (pi/Ne_net)*([1:net_E.N]');  %PO's of network cells

trains = generate_spike_trains(Ne_net,net_E.finp,T/1000);
trains(:,1) = trains(:,1) * 1000;  %convert to ms

%Construct weight matrices
Ginp = .0035;  %mS/cm^2


GEtoE = Mee + G;
GEtoE = GEtoE - spdiags(diag(GEtoE),0,Ne_net,Ne_net);   %no cell synapses onto itself

vthreshold = 0;    % threshold for firing (mV)
net_E.v(:) = v0;          % resting membrane potential (mV)
net_E.gE = net_E.gE*0;
net_E.spks = [];

%Initiate values for external spikes
ct = 1;
spk1 = ee;

%Variables needed for spike counting
aboveE = ee;
spkcnt = ee;

j = 1;
while j*dt < T

    %Find network neurons receiving lgn input
    ind = 0;
    spk1 = 0*spk1;
    while trains(ct,1) < j*dt
        ind = ind + 1;
        spk1(ind) = trains(ct,2);
        ct = ct + 1;
    end
    lgn_spks = spk1(1:ind);  %indeces of net neurons receiving lgn input

    %Values needed for I_active
    alpha_mE = -0.1*(net_E.v+30) ./ (exp(-0.1*(net_E.v+30))-1);
    beta_mE = 4*exp(-(net_E.v+55) / 18);
    alpha_hE = 0.07*exp(-(net_E.v+44) / 20);
    beta_hE = 1 ./ (exp(-0.1*(net_E.v+14))+1);
    alpha_nE = -0.01*(net_E.v+34) ./ (exp(-0.1*(net_E.v+34))-1);
    beta_nE = 0.125*exp(-(net_E.v+44) / 80);
    a_infE = 1 ./ (exp(-(net_E.v+50)/20)+1);
    b_infE = 1 ./ (exp((net_E.v+80)/6)+1);
    m_infE = alpha_mE ./ (alpha_mE+beta_mE);
    tau_hE = phi ./ (alpha_hE+beta_hE);
    h_infE = alpha_hE ./ (alpha_hE+beta_hE);
    tau_nE = phi ./ (alpha_nE+beta_nE);
    n_infE = alpha_nE ./ (alpha_nE+beta_nE);

    if i == 1  %initial conditions
        net_E.h = h_infE;
        net_E.n = n_infE;
        net_E.b = b_infE;
    end

    net_E.m = m_infE;
    net_E.h = (net_E.h .* tau_hE + dt*h_infE) ./ (tau_hE+dt);  % hybrid euler
    net_E.n = (net_E.n .* tau_nE + dt*n_infE) ./ (tau_nE+dt);
    net_E.b = (net_E.b*tau_A + dt*b_infE) / (tau_A+dt);

    %Conductances decay
    net_E.gE = net_E.gE / (1+(dt/tau));

    %Add synaptic input to conductances
    net_E.gE = net_E.gE + sum(GEtoE(:,net_E.spks),2);
    net_E.gE(lgn_spks) = net_E.gE(lgn_spks) + Ginp;

    % Update net_E.v
    tm1 = g_L*E_L;  %I_leak tm

    tm2 = gbar_na * (m_infE .^ 3) .* net_E.h * E_na;  %I_active tms
    tm3 = gbar_k * (net_E.n .^ 4) * E_k;
    tm4 = gbar_A * (a_infE .^ 3) .*net_E.b * E_k;

    top = net_E.v + (dt/Cm) * (tm1 + tm2 + tm3 + tm4);

    tm1 = g_L;  %I_leak tm

    tm2 = gbar_na * (m_infE .^ 3) .* net_E.h;  %I_active tms
    tm3 = gbar_k * (net_E.n .^ 4);
    tm4 = gbar_A * (a_infE .^ 3) .* net_E.b;

    tm5 = net_E.gE;  %I_net tm

    bottom = 1 + (dt/Cm) * (tm1 + tm2 + tm3 + tm4 + tm5);
    net_E.v = top ./ bottom;





    %Check for spikes
    belowE = 1*(net_E.v < vthreshold);
    prod = belowE .* aboveE;  %spk if i was above and is now below
    spkcnt = spkcnt + prod;
    net_E.spks = find(prod);
    aboveE = 1*(net_E.v >= vthreshold);

    j = j + 1;

end

spkfreq = sum(spkcnt)/(T*Ne_net) * 1000;  %Hz

